@@ -30,7 +30,7 @@
 
 #ifndef lint
 static const char rcsid[] _U_ =
-    "@(#) $Header$ (LBL)";
+    "@(#) $Header: /tcpdump/master/libpcap/savefile.c,v 1.126.2.13 2005/08/29 21:05:45 guy Exp $ (LBL)";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -52,7 +52,7 @@ static const char rcsid[] _U_ =
 /*
  * Standard libpcap format.
  */
-#define TCPDUMP_MAGIC 0xa1b2c3d4
+#define TCPDUMP_MAGIC		0xa1b2c3d4
 
 /*
  * Alexey Kuznetzov's modified libpcap format.
@@ -71,6 +71,12 @@ static const char rcsid[] _U_ =
  */
 #define NAVTEL_TCPDUMP_MAGIC	0xa12b3c4d
 
+/*
+ * Normal libpcap format, except for seconds/nanoseconds timestamps,
+ * as per a request by Ulf Lamping <ulf.lamping@web.de>
+ */
+#define NSEC_TCPDUMP_MAGIC	0xa1b23c4d
+
 /*
  * We use the "receiver-makes-right" approach to byte order,
  * because time is at a premium when we are writing the file.
@@ -412,6 +418,18 @@ static const char rcsid[] _U_ =
  */
 #define LINKTYPE_LINUX_LAPD	177
 
+/*
+ * Juniper-private data link type, as per request from
+ * Hannes Gredler <hannes@juniper.net>. 
+ * The Link Types are used for prepending meta-information
+ * like interface index, interface name
+ * before standard Ethernet, PPP, Frelay & C-HDLC Frames
+ */
+#define LINKTYPE_JUNIPER_ETHER  178
+#define LINKTYPE_JUNIPER_PPP    179
+#define LINKTYPE_JUNIPER_FRELAY 180
+#define LINKTYPE_JUNIPER_CHDLC  181
+
 static struct linktype_map {
 	int	dlt;
 	int	linktype;
@@ -576,8 +594,8 @@ static struct linktype_map {
 	 *	LINKTYPE_* values, either).
 	 */
 
-        /* Juniper-internal chassis encapsulation */
-        { DLT_JUNIPER_MONITOR,     LINKTYPE_JUNIPER_MONITOR },
+	/* Juniper-internal chassis encapsulation */
+	{ DLT_JUNIPER_MONITOR,	LINKTYPE_JUNIPER_MONITOR },
 
 	/* BACnet MS/TP */
 	{ DLT_BACNET_MS_TP,	LINKTYPE_BACNET_MS_TP },
@@ -611,6 +629,13 @@ static struct linktype_map {
 	/* viSDN LAPD */
 	{ DLT_LINUX_LAPD,	LINKTYPE_LINUX_LAPD },
 
+        /* Juniper meta-information before Ether, PPP, Frame Relay, C-HDLC Frames */
+        { DLT_JUNIPER_ETHER, LINKTYPE_JUNIPER_ETHER },
+        { DLT_JUNIPER_PPP, LINKTYPE_JUNIPER_PPP },
+        { DLT_JUNIPER_FRELAY, LINKTYPE_JUNIPER_FRELAY },
+        { DLT_JUNIPER_CHDLC, LINKTYPE_JUNIPER_CHDLC },
+
+
 	{ -1,			-1 }
 };
 
@@ -803,7 +828,7 @@ pcap_fopen_offline(FILE *fp, char *errbuf)
 			    "truncated dump file; tried to read %lu file header bytes, only got %lu",
 			    (unsigned long)sizeof(hdr),
 			    (unsigned long)amt_read);
-	}
+		}
 		goto bad;
 	}
 	magic = hdr.magic;
