@@ -3,23 +3,23 @@
  * nbase.h -- The main include file exposing the external API for          *
  * libnbase, a library of base (often compatability) routines.  Programs   *
  * using libnbase can guarantee the availability of functions like         *
- * (v)snprintf and inet_aton.  This library also provides consistency and  *
+ * (v)snprintf and inet_pton.  This library also provides consistency and  *
  * extended features for some functions.  It was originally written for    *
  * use in the Nmap Security Scanner ( http://www.insecure.org/nmap/ ).     *
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
- * is also a registered trademark of Insecure.Com LLC.  This program is    *
- * free software; you may redistribute and/or modify it under the          *
- * terms of the GNU General Public License as published by the Free        *
- * Software Foundation; Version 2.  This guarantees your right to use,     *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we may be  *
- * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as  *
- * our remote OS fingerprinting database and code, service/version         *
- * detection system, and port scanning code.                               *
+ * The Nmap Security Scanner is (C) 1996-2006 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -42,7 +42,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://www.insecure.org/nmap/ to download Nmap.                         *
+ * http://insecure.org/nmap/ to download Nmap.                             *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -54,10 +54,10 @@
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to many    *
- * security vendors, and generally include a perpetual license as well as  *
- * providing for priority support and updates as well as helping to fund   *
- * the continued development of Nmap technology.  Please email             *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
  * sales@insecure.com for further information.                             *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
@@ -101,7 +101,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nbase.h 2966 2005-12-05 21:31:10Z fyodor $ */
+/* $Id: nbase.h 3899 2006-08-29 05:42:35Z fyodor $ */
 
 #ifndef NBASE_H
 #define NBASE_H
@@ -112,7 +112,7 @@
  * * 'inline' is defined to what is neccessary for the C compiler being
  *   used (which may be nothing)
  *
- * * snprintf, inet_aton, inet_pton, inet_aton, memcpy, and bzero are 
+ * * snprintf, inet_pton, memcpy, and bzero are 
  *   provided if you don't have them (prototypes for these are 
  *   included either way).
  *
@@ -311,7 +311,7 @@ extern "C" int vsnprintf (char *, size_t, const char *, va_list);
 
 #define strcasecmp _stricmp
 #define strncasecmp _strnicmp
-
+#define execv _execv
 
 #endif /* WIN32 */
 
@@ -369,11 +369,6 @@ int gettimeofday(struct timeval *tv, struct timeval *tz);
 unsigned int sleep(unsigned int seconds);
 #endif
 
-#ifndef HAVE_INET_ATON
-struct in_addr;
-int inet_aton(const char *cp, struct in_addr *addr);
-#endif
-
 /* Strncpy is like strcpy() except it ALWAYS zero-terminates, even if
    it must truncate */
 int Strncpy(char *dest, const char *src, size_t n);
@@ -388,10 +383,10 @@ void replacenonprintable(char *str, int strlength, char replchar);
 
 /* A few simple wrappers for the most common memory allocation routines which will exit() if the
 	allocation fails, so you don't always have to check -- see nbase_memalloc.c */
-void *safe_malloc(int size);
+void *safe_malloc(size_t size);
 void *safe_realloc(void *ptr, size_t size);
 /* Zero-initializing version of safe_malloc */
-void *safe_zalloc(int size);
+void *safe_zalloc(size_t size);
 
   /* Some routines for obtaining simple (not secure on systems that
      lack /dev/random and friends' "random" numbers */
@@ -403,6 +398,8 @@ u32 get_random_u32();
 u16 get_random_u16();
 u8 get_random_u8();
 
+/* CRC16 Cyclic Redundancy Check  */
+unsigned long crc16(unsigned char *buf, int len);
 
 #ifndef STDIN_FILENO
 #define STDIN_FILENO 0
