@@ -8,17 +8,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
- * is also a registered trademark of Insecure.Com LLC.  This program is    *
- * free software; you may redistribute and/or modify it under the          *
- * terms of the GNU General Public License as published by the Free        *
- * Software Foundation; Version 2.  This guarantees your right to use,     *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we may be  *
- * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as  *
- * our remote OS fingerprinting database and code, service/version         *
- * detection system, and port scanning code.                               *
+ * The Nmap Security Scanner is (C) 1996-2006 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -41,7 +41,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://www.insecure.org/nmap/ to download Nmap.                         *
+ * http://insecure.org/nmap/ to download Nmap.                             *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -53,10 +53,10 @@
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to many    *
- * security vendors, and generally include a perpetual license as well as  *
- * providing for priority support and updates as well as helping to fund   *
- * the continued development of Nmap technology.  Please email             *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
  * sales@insecure.com for further information.                             *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
@@ -100,7 +100,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nbase_memalloc.c 2443 2004-10-12 09:34:12Z fyodor $ */
+/* $Id: nbase_memalloc.c 3899 2006-08-29 05:42:35Z fyodor $ */
 
 #include "nbase.h"
 #include <stdio.h>
@@ -118,10 +118,10 @@ va_end(ap);
 exit(1);
 }
 
-void *safe_malloc(int size)
+void *safe_malloc(size_t size)
 {
   void *mymem;
-  if (size < 0)
+  if ((int) size < 0)  /* Catch caller errors */
     fatal("Tried to malloc negative amount of memory!!!");
   mymem = malloc(size);
   if (mymem == NULL)
@@ -132,7 +132,7 @@ void *safe_malloc(int size)
 void *safe_realloc(void *ptr, size_t size)
 {
   void *mymem;
-  if (size < 0)
+  if ((int) size < 0) /* Catch caller errors */
     fatal("Tried to realloc negative amount of memory!!!");
   mymem = realloc(ptr, size);
   if (mymem == NULL)
@@ -141,10 +141,10 @@ void *safe_realloc(void *ptr, size_t size)
 }
 
 /* Zero-initializing version of safe_malloc */
-void *safe_zalloc(int size)
+void *safe_zalloc(size_t size)
 {
   void *mymem;
-  if (size < 0)
+  if ((int) size < 0)
     fatal("Tried to malloc negative amount of memory!!!");
   mymem = calloc(1, size);
   if (mymem == NULL)
