@@ -5,15 +5,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1995-2003 Insecure.Com LLC. This       *
- * program is free software; you may redistribute and/or modify it under   *
- * the terms of the GNU General Public License as published by the Free    *
- * Software Foundation; Version 2.  This guarantees your right to use,     *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we may be  *
- * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as   *
- * our remote OS fingerprinting database and code.                         *
+ * The Nmap Security Scanner is (C) 1996-2006 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -23,8 +25,11 @@
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
  *   nmap-os-fingerprints or nmap-service-probes.                          *
- * o Executes Nmap                                                         *
- * o Integrates/includes/aggregates Nmap into an executable installer      *
+ * o Executes Nmap and parses the results (as opposed to typical shell or  *
+ *   execution-menu apps, which simply display raw Nmap output and so are  *
+ *   not derivative works.)                                                * 
+ * o Integrates/includes/aggregates Nmap into a proprietary executable     *
+ *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
@@ -33,7 +38,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://www.insecure.org/nmap/ to download Nmap.                         *
+ * http://insecure.org/nmap/ to download Nmap.                             *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -45,14 +50,23 @@
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to many    *
- * security vendors, and generally include a perpetual license as well as  *
- * providing for priority support and updates as well as helping to fund   *
- * the continued development of Nmap technology.  Please email             *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
  * sales@insecure.com for further information.                             *
  *                                                                         *
+ * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * combinations including the two. You must obey the GNU GPL in all        *
+ * respects for all of the code used other than OpenSSL.  If you modify    *
+ * this file, you may extend this exception to your version of the file,   *
+ * but you are not obligated to do so.                                     *
+ *                                                                         *
  * If you received these files with a written license agreement or         *
- * contract stating terms other than the (GPL) terms above, then that      *
+ * contract stating terms other than the terms above, then that            *
  * alternative license agreement takes precedence over these comments.     *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
@@ -78,11 +92,12 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html .                                  *
+ * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
+ * with Nmap.                                                              *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: strcasecmp.c,v 1.7 2003/09/11 02:12:57 fyodor Exp $ */
+/* $Id: strcasecmp.c 3899 2006-08-29 05:42:35Z fyodor $ */
 
 #if !defined(HAVE_STRCASECMP) || !defined(HAVE_STRNCASECMP)
 #include <stdlib.h>
@@ -93,65 +108,46 @@
 #ifndef HAVE_STRCASECMP
 int strcasecmp(const char *s1, const char *s2)
 {
-  int ret;
+	int i, ret;
+	char *cp1, *cp2;
   
-  char *cp1,*cp2;
-  int i=0;
-  
-  cp1=malloc(strlen(s1)+1);
-  memset(cp1,0,strlen(s1)+1);
-  memcpy(cp1,s1,strlen(s1));
-  for (i=0; cp1[i]>0; i++)
-    {
-      if ('a' <= cp1[i] && cp1[i] <= 'z')
-	cp1[i] -= 32;
-    }
-  
-  cp2=malloc(strlen(s2)+1);
-  memset(cp2,0,strlen(s2)+1);
-  memcpy(cp2,s2,strlen(s2));
-  for (i=0; cp2[i]>0; i++)
-    {
-      if ('a' <= cp2[i] && cp2[i] <= 'z')
-	cp2[i] -= 32;
-    }
+	cp1 = malloc(strlen(s1) + 1);
+	cp2 = malloc(strlen(s2) + 1);
 
-  ret=strcmp(cp1,cp2);
+	for (i = 0; i < strlen(s1) + 1; i++)
+		cp1[i] = tolower(s1[i]);
+	for (i = 0; i < strlen(s2) + 1; i++)
+		cp2[i] = tolower(s2[i]);
   
-  free(cp1);
-  free(cp2);
-  return ret;
+	ret = strcmp(cp1, cp2);
+  
+	free(cp1);
+	free(cp2);
+
+	return ret;
 }
 #endif
 
 #ifndef HAVE_STRNCASECMP
 int strncasecmp(const char *s1, const char *s2, size_t n)
 {
-  int ret;
-  char *cp1,*cp2;
-  int i=0;
+	int i, ret;
+	char *cp1, *cp2;
   
-  cp1= (char *) malloc(strlen(s1)+1);
-  memset(cp1,0,strlen(s1)+1);
-  memcpy(cp1,s1,strlen(s1));
-  for (i=0; cp1[i]>0; i++)
-    {
-      if ('a' <= cp1[i] && cp1[i] <= 'z')
-	cp1[i] -= 32;
-    }
+	cp1 = malloc(strlen(s1) + 1);
+	cp2 = malloc(strlen(s2) + 1);
+
+	for (i = 0; i < strlen(s1) + 1; i++)
+		cp1[i] = tolower(s1[i]);
+	for (i = 0; i < strlen(s2) + 1; i++)
+		cp2[i] = tolower(s2[i]);
   
-  cp2= (char *) malloc(strlen(s2)+1);
-  memset(cp2,0,strlen(s2)+1);
-  memcpy(cp2,s2,strlen(s2));
-  for (i=0; cp2[i]>0; i++)
-    {
-      if ('a' <= cp2[i] && cp2[i] <= 'z')
-	cp2[i] -= 32;
-    }
+	ret = strncmp(cp1, cp2, n);
   
-  ret=strncmp(cp1,cp2,n);
-  free(cp1);
-  free(cp2);
-  return ret;
+	free(cp1);
+	free(cp2);
+
+	return ret;
 }
 #endif
+
