@@ -5,7 +5,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2003 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2006 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -14,6 +14,15 @@
  * may be willing to sell alternative licenses (contact                    *
  * sales@insecure.com ).                                                   *
  *                                                                         *
+ * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * combinations including the two. You must obey the GNU GPL in all        *
+ * respects for all of the code used other than OpenSSL.  If you modify    *
+ * this file, you may extend this exception to your version of the file,   *
+ * but you are not obligated to do so.                                     *
+ *                                                                         * 
  * If you received these files with a written license agreement stating    *
  * terms other than the (GPL) terms above, then that alternative license   *
  * agreement takes precedence over this comment.                          *
@@ -45,7 +54,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nsock.h,v 1.20 2003/09/16 06:04:55 fyodor Exp $ */
+/* $Id: nsock.h 3870 2006-08-25 01:47:53Z fyodor $ */
 
 #ifndef NSOCK_H
 #define NSOCK_H
@@ -55,7 +64,7 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <time.h>
+#include <sys/time.h>
 #endif
 
 #ifdef __cplusplus
@@ -391,6 +400,14 @@ nsock_event_id nsock_connect_ssl(nsock_pool nsp, nsock_iod nsiod,
 				 size_t sslen, unsigned short port,
 				 nsock_ssl_session ssl_session);
 
+/* Request ssl connection over already established TCP connection.
+   nsiod must be socket that is already connected to target
+   using nsock_connect_tcp.
+   All parameters have the same meaning as in 'nsock_connect_ssl' */
+nsock_event_id nsock_reconnect_ssl(nsock_pool nsp, nsock_iod nsiod,
+				   nsock_ev_handler handler, int timeout_msecs,
+                                   void *userdata, nsock_ssl_session ssl_session);
+
 /* Read up to nlines lines (terminated with \n, which of course
    inclues \r\n), or until EOF, or until the timeout, whichever comes
    first.  Note that NSE_STATUS_SUCCESS will be returned in the case
