@@ -6,7 +6,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2003 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2006 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -15,6 +15,15 @@
  * may be willing to sell alternative licenses (contact                    *
  * sales@insecure.com ).                                                   *
  *                                                                         *
+ * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * combinations including the two. You must obey the GNU GPL in all        *
+ * respects for all of the code used other than OpenSSL.  If you modify    *
+ * this file, you may extend this exception to your version of the file,   *
+ * but you are not obligated to do so.                                     *
+ *                                                                         * 
  * If you received these files with a written license agreement stating    *
  * terms other than the (GPL) terms above, then that alternative license   *
  * agreement takes precedence over this comment.                          *
@@ -46,7 +55,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nsock_internal.h,v 1.15 2003/09/13 04:19:56 fyodor Exp $ */
+/* $Id: nsock_internal.h 3870 2006-08-25 01:47:53Z fyodor $ */
 
 #ifndef NSOCK_INTERNAL_H
 #define NSOCK_INTERNAL_H
@@ -90,7 +99,7 @@
 #include <strings.h>
 #endif
 
-#include "utils.h"
+#include "nsock_utils.h"
 
 /*********      STRUCTURES   **************/
 
@@ -105,9 +114,12 @@ struct nsock_io_info {
   /* For keeping track of the select results */
   fd_set fds_results_r, fds_results_w, fds_results_x;
 
-  int max_sd; /* The highest sd we have set in any of our fd_set's (max_sd + 1 is used in select() calls) */
+/* The highest sd we have set in any of our fd_set's (max_sd + 1 is
+   used in select() calls).  Note that it can be -1, when there are no
+   valid sockets */
+  int max_sd; 
   int results_left; /* The number of descriptors contained in the sets that
-		       we have not yet dealt with. Not currently used. */
+		       we have not yet dealt with. */
 };
 
 struct event_lists {
