@@ -6,17 +6,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
- * is also a registered trademark of Insecure.Com LLC.  This program is    *
- * free software; you may redistribute and/or modify it under the          *
- * terms of the GNU General Public License as published by the Free        *
- * Software Foundation; Version 2.  This guarantees your right to use,     *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we may be  *
- * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as  *
- * our remote OS fingerprinting database and code, service/version         *
- * detection system, and port scanning code.                               *
+ * The Nmap Security Scanner is (C) 1996-2006 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -39,7 +39,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://www.insecure.org/nmap/ to download Nmap.                         *
+ * http://insecure.org/nmap/ to download Nmap.                             *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -51,10 +51,10 @@
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to many    *
- * security vendors, and generally include a perpetual license as well as  *
- * providing for priority support and updates as well as helping to fund   *
- * the continued development of Nmap technology.  Please email             *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
  * sales@insecure.com for further information.                             *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
@@ -98,7 +98,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: service_scan.cc 3516 2006-06-19 04:19:52Z fyodor $ */
+/* $Id: service_scan.cc 3977 2006-09-12 03:35:29Z fyodor $ */
 
 
 #include "service_scan.h"
@@ -255,10 +255,10 @@ struct substargs {
 };
 
 /********************   PROTOTYPES *******************/
-void servicescan_read_handler(nsock_pool nsp, nsock_event nse, void *mydata);
-void servicescan_write_handler(nsock_pool nsp, nsock_event nse, void *mydata);
-void servicescan_connect_handler(nsock_pool nsp, nsock_event nse, void *mydata);
-void end_svcprobe(nsock_pool nsp, enum serviceprobestate probe_state, ServiceGroup *SG, ServiceNFO *svc, nsock_iod nsi);
+static void servicescan_read_handler(nsock_pool nsp, nsock_event nse, void *mydata);
+static void servicescan_write_handler(nsock_pool nsp, nsock_event nse, void *mydata);
+static void servicescan_connect_handler(nsock_pool nsp, nsock_event nse, void *mydata);
+static void end_svcprobe(nsock_pool nsp, enum serviceprobestate probe_state, ServiceGroup *SG, ServiceNFO *svc, nsock_iod nsi);
 
 ServiceProbeMatch::ServiceProbeMatch() {
   deflineno = -1;
@@ -305,6 +305,7 @@ void ServiceProbeMatch::InitMatch(const char *matchtext, int lineno) {
   const char *pcre_errptr = NULL;
   int pcre_erroffset = 0;
   unsigned int tmpbuflen = 0;
+  char **curr_tmp = NULL;
 
   if (isInitialized) fatal("Sorry ... ServiceProbeMatch::InitMatch does not yet support reinitializion");
   if (!matchtext || !*matchtext) 
@@ -414,13 +415,27 @@ void ServiceProbeMatch::InitMatch(const char *matchtext, int lineno) {
       tmptemplate[tmpbuflen] = '\0';
     }
 
-    if (modechar == 'p') product_template = tmptemplate;
-    else if (modechar == 'v') version_template = tmptemplate;
-    else if (modechar == 'i') info_template = tmptemplate;
-    else if (modechar == 'h') hostname_template = tmptemplate;
-    else if (modechar == 'o') ostype_template = tmptemplate;
-    else if (modechar == 'd') devicetype_template = tmptemplate;
-    else fatal("ServiceProbeMatch::InitMatch: Unknown template specifier '%c' on line %d of nmap-service-probes", modechar, lineno);
+    switch(modechar){
+    case 'p': curr_tmp = &product_template; break;
+    case 'v': curr_tmp = &version_template; break;
+    case 'i': curr_tmp = &info_template; break;
+    case 'h': curr_tmp = &hostname_template; break;
+    case 'o': curr_tmp = &ostype_template; break;
+    case 'd': curr_tmp = &devicetype_template; break;
+    default:
+    	fatal("ServiceProbeMatch::InitMatch: Unknown template specifier '%c' on line %d of nmap-service-probes", modechar, lineno);
+    }
+    if(*curr_tmp){
+      if(o.debugging)
+        error("WARNING: Template \"%c/%s/\" replaced with \"%c/%s/\" on line %d of nmap-service-probes",
+        	modechar,
+    	  	*curr_tmp,
+        	modechar,
+    	  	tmptemplate,
+    	  	lineno);
+    	free(*curr_tmp);
+      }
+    *curr_tmp = tmptemplate;
 
     matchtext = p + 1;
   }
@@ -968,7 +983,7 @@ void ServiceProbe::setPortVector(vector<u16> *portv, const char *portstr,
   // SERVICE_TUNNEL_SSL.  Otherwise use SERVICE_TUNNEL_NONE.  The line
   // number is requested because this function will bail with an error
   // (giving the line number) if it fails to parse the string.  Ports
-  // are a comma seperated list of ports and ranges
+  // are a comma separated list of ports and ranges
   // (e.g. 53,80,6000-6010).
 void ServiceProbe::setProbablePorts(enum service_tunnel_type tunnel,
 				    const char *portstr, int lineno) {
@@ -1037,9 +1052,9 @@ void ServiceProbe::addMatch(const char *match, int lineno) {
   matches.push_back(newmatch);
 }
 
-// Parses the given nmap-service-probes file into the AP class
-/* Must NOT be static because I have externam maintenance tools (servicematch)
-  which use this */
+/* Parses the given nmap-service-probes file into the AP class Must
+   NOT be made static because I have external maintenance tools
+   (servicematch) which use this */
 void parse_nmap_service_probe_file(AllProbes *AP, char *filename) {
   ServiceProbe *newProbe;
   char line[2048];
@@ -1123,7 +1138,7 @@ void parse_nmap_service_probe_file(AllProbes *AP, char *filename) {
 
 // Parses the nmap-service-probes file, and adds each probe to
 // the already-created 'probes' vector.
-void parse_nmap_service_probes(AllProbes *AP) {
+static void parse_nmap_service_probes(AllProbes *AP) {
   char filename[256];
 
   if (nmap_fetchfile(filename, sizeof(filename), "nmap-service-probes") == -1){
@@ -1133,17 +1148,26 @@ void parse_nmap_service_probes(AllProbes *AP) {
   parse_nmap_service_probe_file(AP, filename);
 }
 
-static AllProbes *service_scan_init(void)
+AllProbes *AllProbes::global_AP;
+AllProbes *AllProbes::service_scan_init(void)
 {
-  static AllProbes *AP;
+  if(global_AP)
+    return global_AP;
+  global_AP = new AllProbes();
+  parse_nmap_service_probes(global_AP);
 
-  if (AP) return AP;
-  AP = new AllProbes();
-  parse_nmap_service_probes(AP);
-
-  return AP;
+  return global_AP;
 }
 
+void AllProbes::service_scan_free(void)
+{
+  if(global_AP){
+    delete global_AP;
+    global_AP = NULL;
+  }
+}
+
+
 // If the buf (of length buflen) matches one of the regexes in this
 // ServiceProbe, returns the details of the match (service name,
 // version number if applicable, and whether this is a "soft" match.
@@ -1175,8 +1199,12 @@ AllProbes::~AllProbes() {
   vector<ServiceProbe *>::iterator vi;
 
   // Delete all the ServiceProbe's inside the probes vector
-  for(vi = probes.begin(); vi != probes.end(); vi++)
+  for(vi = probes.begin(); vi != probes.end(); vi++) {
     delete *vi;
+  }
+  if(nullProbe)
+    delete nullProbe;
+  free_scan_lists(excludedports);
 }
 
   // Tries to find the probe in this AllProbes class which have the
@@ -1865,7 +1893,7 @@ static void handleHostIfDone(ServiceGroup *SG, Target *target) {
 // A simple helper function to cancel further work on a service and
 // set it to the given probe_state pass NULL for nsi if you don't want
 // it to be deleted (for example, if you already have done so).
-void end_svcprobe(nsock_pool nsp, enum serviceprobestate probe_state, ServiceGroup *SG, ServiceNFO *svc, nsock_iod nsi) {
+static void end_svcprobe(nsock_pool nsp, enum serviceprobestate probe_state, ServiceGroup *SG, ServiceNFO *svc, nsock_iod nsi) {
   list<ServiceNFO *>::iterator member;
   Target *target = svc->target;
 
@@ -1954,7 +1982,7 @@ static int launchSomeServiceProbes(nsock_pool nsp, ServiceGroup *SG) {
 }
 
 
-void servicescan_connect_handler(nsock_pool nsp, nsock_event nse, void *mydata) {
+static void servicescan_connect_handler(nsock_pool nsp, nsock_event nse, void *mydata) {
   nsock_iod nsi = nse_iod(nse);
   enum nse_status status = nse_status(nse);
   enum nse_type type = nse_type(nse);
@@ -2014,7 +2042,7 @@ void servicescan_connect_handler(nsock_pool nsp, nsock_event nse, void *mydata)
   return;
 }
 
-void servicescan_write_handler(nsock_pool nsp, nsock_event nse, void *mydata) {
+static void servicescan_write_handler(nsock_pool nsp, nsock_event nse, void *mydata) {
   enum nse_status status = nse_status(nse);
   nsock_iod nsi;
   ServiceNFO *svc = (ServiceNFO *)mydata;
@@ -2064,7 +2092,7 @@ void servicescan_write_handler(nsock_pool nsp, nsock_event nse, void *mydata) {
   return;
 }
 
-void servicescan_read_handler(nsock_pool nsp, nsock_event nse, void *mydata) {
+static void servicescan_read_handler(nsock_pool nsp, nsock_event nse, void *mydata) {
   nsock_iod nsi = nse_iod(nse);
   enum nse_status status = nse_status(nse);
   enum nse_type type = nse_type(nse);
@@ -2338,7 +2366,7 @@ int service_scan(vector<Target *> &Targets) {
   if (Targets.size() == 0)
     return 1;
 
-  AP = service_scan_init();
+  AP = AllProbes::service_scan_init();
 
 
   // Now I convert the targets into a new ServiceGroup
@@ -2361,16 +2389,15 @@ int service_scan(vector<Target *> &Targets) {
   starttime = time(NULL);
   if (o.verbose) {
     char targetstr[128];
-    struct tm *tm = localtime(&starttime);
     bool plural = (Targets.size() != 1);
     if (!plural) {
       (*(Targets.begin()))->NameIP(targetstr, sizeof(targetstr));
     } else snprintf(targetstr, sizeof(targetstr), "%u hosts", (unsigned) Targets.size());
 
-    log_write(LOG_STDOUT, "Initiating service scan against %u %s on %s at %02d:%02d\n", 
+    log_write(LOG_STDOUT, "Scanning %u %s on %s\n", 
 	      (unsigned) SG->services_remaining.size(), 
 	      (SG->services_remaining.size() == 1)? "service" : "services", 
-	      targetstr, tm->tm_hour, tm->tm_min);
+	      targetstr);
   }
 
   // Lets create a nsock pool for managing all the concurrent probes
@@ -2399,18 +2426,16 @@ int service_scan(vector<Target *> &Targets) {
   nsp_delete(nsp);
 
   if (o.verbose) {
-    gettimeofday(&now, NULL);
+    char additional_info[128];
     if (SG->num_hosts_timedout == 0)
-      log_write(LOG_STDOUT, "The service scan took %.2fs to scan %u %s on %u %s.\n", 
-		TIMEVAL_MSEC_SUBTRACT(now, starttv) / 1000.0, 
+      snprintf(additional_info, sizeof(additional_info), "%u %s on %u %s",
 		(unsigned) SG->services_finished.size(),  
 		(SG->services_finished.size() == 1)? "service" : "services", 
 		(unsigned) Targets.size(), (Targets.size() == 1)? "host" : "hosts");
-    else log_write(LOG_STDOUT, 
-		   "Finished service scan in %.2fs, but %d %s timed out.\n", 
-		   TIMEVAL_MSEC_SUBTRACT(now, starttv) / 1000.0, 
+    else snprintf(additional_info, sizeof(additional_info), "%u %s timed out", 
 		   SG->num_hosts_timedout, 
 		   (SG->num_hosts_timedout == 1)? "host" : "hosts");
+    SG->SPM->endTask(NULL, additional_info);
   }
 
   // Yeah - done with the service scan.  Now I go through the results
