@@ -6,17 +6,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
- * is also a registered trademark of Insecure.Com LLC.  This program is    *
- * free software; you may redistribute and/or modify it under the          *
- * terms of the GNU General Public License as published by the Free        *
- * Software Foundation; Version 2.  This guarantees your right to use,     *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we may be  *
- * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as  *
- * our remote OS fingerprinting database and code, service/version         *
- * detection system, and port scanning code.                               *
+ * The Nmap Security Scanner is (C) 1996-2006 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -39,7 +39,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://www.insecure.org/nmap/ to download Nmap.                         *
+ * http://insecure.org/nmap/ to download Nmap.                             *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -51,10 +51,10 @@
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to many    *
- * security vendors, and generally include a perpetual license as well as  *
- * providing for priority support and updates as well as helping to fund   *
- * the continued development of Nmap technology.  Please email             *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
  * sales@insecure.com for further information.                             *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
@@ -98,7 +98,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: targets.cc 3274 2006-04-22 23:00:09Z fyodor $ */
+/* $Id: targets.cc 4198 2006-11-19 09:35:26Z fyodor $ */
 
 
 #include "targets.h"
@@ -794,7 +794,6 @@ static int get_ping_results(int sd, pcap_t *pd, Target *hostbatch[],
   return 0;
 }
 
-
 static int sendconnecttcpquery(Target *hostbatch[], struct tcpqueryinfo *tqi,
 			Target *target, int probe_port_num, u16 seq, 
 			struct timeval *time, struct pingtune *pt, 
@@ -888,8 +887,8 @@ static int sendconnecttcpqueries(Target *hostbatch[], struct tcpqueryinfo *tqi,
   return 0;
 }
 
-int sendrawudppingquery(int rawsd, struct eth_nfo *eth, Target *target, u16 probe_port,
-			u16 seq, struct timeval *time, struct pingtune *pt) {
+static int sendrawudppingquery(int rawsd, struct eth_nfo *eth, Target *target, u16 probe_port,
+			       u16 seq, struct timeval *time, struct pingtune *pt) {
 int trynum = 0;
 unsigned short sportbase;
 
@@ -901,14 +900,18 @@ else {
 
  o.decoys[o.decoyturn].s_addr = target->v4source().s_addr;
  
- send_udp_raw_decoys( rawsd, eth, target->v4hostip(), o.ttl, sportbase + trynum, probe_port, seq, o.extra_payload, o.extra_payload_length);
+ send_udp_raw_decoys( rawsd, eth, target->v4hostip(),
+ 		      o.ttl, seq,
+ 		      o.ipoptions, o.ipoptionslen,
+ 		      sportbase + trynum, probe_port, 
+ 		      o.extra_payload, o.extra_payload_length);
 
 
  return 0;
 }
 
-int sendrawtcppingquery(int rawsd, struct eth_nfo *eth, Target *target, int pingtype, u16 probe_port,
-			u16 seq, struct timeval *time, struct pingtune *pt) {
+static int sendrawtcppingquery(int rawsd, struct eth_nfo *eth, Target *target, int pingtype, u16 probe_port,
+			       u16 seq, struct timeval *time, struct pingtune *pt) {
 int trynum = 0;
 int myseq;
 unsigned short sportbase;
@@ -925,18 +928,28 @@ else {
  o.decoys[o.decoyturn].s_addr = target->v4source().s_addr;
 
  if (pingtype & PINGTYPE_TCP_USE_SYN) {   
-   send_tcp_raw_decoys( rawsd, eth, target->v4hostip(), o.ttl, sportbase + trynum, probe_port, myseq, myack, TH_SYN, 0, (u8 *) "\x02\x04\x05\xb4", 4, o.extra_payload, 
-			o.extra_payload_length);
+   send_tcp_raw_decoys( rawsd, eth, target->v4hostip(),
+   			o.ttl, false,
+   			o.ipoptions, o.ipoptionslen,
+   			sportbase + trynum, probe_port,
+   			myseq, myack, 0, TH_SYN, 0, 0,
+   			(u8 *) "\x02\x04\x05\xb4", 4,
+   			o.extra_payload, o.extra_payload_length);
  } else {
-   send_tcp_raw_decoys( rawsd, eth, target->v4hostip(), o.ttl, sportbase + trynum, probe_port, myseq, myack, TH_ACK, 0, NULL, 0, o.extra_payload, 
-			o.extra_payload_length);
+   send_tcp_raw_decoys( rawsd, eth, target->v4hostip(),
+   			o.ttl, false,
+   			o.ipoptions, o.ipoptionslen,
+   			sportbase + trynum, probe_port,
+   			myseq, myack, 0, TH_ACK, 0, 0,
+   			NULL, 0,
+   			o.extra_payload, o.extra_payload_length);
  }
 
  return 0;
 }
 
-int sendrawtcpudppingqueries(int rawsd, eth_t *ethsd, Target *target, int pingtype, u16 seq, 
-			  struct timeval *time, struct pingtune *pt) {
+static int sendrawtcpudppingqueries(int rawsd, eth_t *ethsd, Target *target, int pingtype, u16 seq, 
+				    struct timeval *time, struct pingtune *pt) {
   int i;
   struct eth_nfo eth;
   struct eth_nfo *ethptr = NULL;
@@ -1069,7 +1082,11 @@ if (ptech.icmpscan) {
        fprintf(stderr, "sendto: %s\n", strerror(sock_err));
      }
    } else {
-     send_ip_raw( rawsd, ethptr, &o.decoys[decoy], target->v4hostip(), o.ttl, IPPROTO_ICMP, ping, icmplen);
+     send_ip_raw(rawsd, ethptr,
+     		&o.decoys[decoy], target->v4hostip(),
+     		IPPROTO_ICMP, o.ttl,
+     		o.ipoptions, o.ipoptionslen,
+     		ping, icmplen);
    }
  }
 
@@ -1097,10 +1114,10 @@ static int sendpingqueries(int sd, int rawsd, eth_t *ethsd, Target *target,
 }
 
 
-int get_connecttcpscan_results(struct tcpqueryinfo *tqi, 
-			       Target *hostbatch[], 
-			       struct timeval *time, struct pingtune *pt, 
-			       struct timeout_info *to) {
+static int get_connecttcpscan_results(struct tcpqueryinfo *tqi, 
+				      Target *hostbatch[], 
+				      struct timeval *time, struct pingtune *pt, 
+				      struct timeout_info *to) {
 
 int res, res2;
 int tm;
@@ -1539,14 +1556,14 @@ static void massping(Target *hostbatch[], int num_hosts,
       rawsd = -1; rawpingsd = -1;
     } else {
       if ((rawsd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) < 0 )
-	pfatal("socket trobles in massping");
+	pfatal("socket troubles in massping");
       broadcast_socket(rawsd);
 #ifndef WIN32
       sethdrinclude(rawsd);
 #endif
 
       if ((rawpingsd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) < 0 )
-	pfatal("socket trobles in massping");
+	pfatal("socket troubles in massping");
       broadcast_socket(rawpingsd);
 #ifndef WIN32
       sethdrinclude(rawpingsd);
@@ -1562,7 +1579,7 @@ static void massping(Target *hostbatch[], int num_hosts,
        16 bytes of the TCP header
        ---
        = 104 byte snaplen */
-    pd = my_pcap_open_live(hostbatch[0]->deviceName(), 104, o.spoofsource, 20);
+    pd = my_pcap_open_live(hostbatch[0]->deviceName(), 104, o.spoofsource, pcap_selectable_fd_valid()? 200 : 15);
 
     snprintf(filter, sizeof(filter), "(icmp and dst host %s) or ((tcp or udp) and dst host %s and ( dst port %d or dst port %d or dst port %d or dst port %d or dst port %d))", 
 	     inet_ntoa(hostbatch[0]->v4source()),
